@@ -4,12 +4,15 @@ class Base(BaseEstimator, TransformerMixin):
     def __init__(self):
         
         self.exchange_media = "default" # e.g. "default", "kafka"
+        self.bootstrap_servers=[]
+        self.fernet_key=""
         self.broker_host = ""
         self.broker_port = ""
         # can be string or a list
         # if previous transformer is parallelTransformer, then would be a list
         self.pre_node_name = "" 
         self.delete_kafka_topic=1
+        self.pipeline_start_time=""
 
         self.node_id = ""
         self.node_name = ""
@@ -26,9 +29,6 @@ class Base(BaseEstimator, TransformerMixin):
         self.is_first_trans = 0
         self.is_last_trans = 0
 
-        
-
-
     def fit(self):
         pass
     
\ No newline at end of file
