@@ -9,7 +9,8 @@ from cetl.functional_modules.parallel_transformer import parallelTransformer
 from cetl.utils.builder import (build_transformer_from_cfg, get_register, FUNCTIONAL_TRANSFORMERS)
 from .dot_node import update_node, recursive_build_digraph, add_dot_node, init_digraph, append_node_text
 from .file_mgt import get_datetime_str
-
+from cryptography.fernet import Fernet
+import hashlib
 
 def _name_estimators(estimators):
     """Generate names for estimators."""
@@ -56,8 +57,7 @@ def recursive_build_pipeline_from_cfg(  cfg,
                                         index, 
                                         parallel_index, 
                                         nodename2transformer, 
-                                        pipeline_settings,
-                                        kafka_media):
+                                        pipeline_settings):
     """
     Examples
     -----------
@@ -129,9 +129,11 @@ def recursive_build_pipeline_from_cfg(  cfg,
                     # transformer.__name__
                     transformer.description = description
                     transformer.breakpoint = sub_breakpoint
+                    transformer.bootstrap_servers=pipeline_settings.bootstrap_servers
                     transformer.print_task_result = pipeline_settings.print_task_result
                     transformer.exchange_media = pipeline_settings.exchange_media
-                    transformer.kafka_media = kafka_media
+                    transformer.fernet_key = pipeline_settings.fernet_key
+                    transformer.pipe_topic_name = pipeline_settings.pipe_topic_name
                     transformers_list.append(transformer)
                     # update nodename2transformer
                     nodename2transformer[transformer.node_name] = transformer
@@ -158,8 +160,7 @@ def recursive_build_pipeline_from_cfg(  cfg,
                                                                                 index, 
                                                                                 parallel_index, 
                                                                                 nodename2transformer,
-                                                                                pipeline_settings,
-                                                                                kafka_media)
+                                                                                pipeline_settings)
                     transformers_list.append(transformer)
                     parallel_node_pairs[f"process_no.{str(i)}"]=_node_pairs
                     parallel_latest_node.append(_latest_node)
@@ -202,8 +203,7 @@ def recursive_build_pipeline_from_cfg(  cfg,
                                                                         index, 
                                                                         parallel_index, 
                                                                         nodename2transformer,
-                                                                        pipeline_settings,
-                                                                        kafka_media)
+                                                                        pipeline_settings)
             # update steps
             node_pairs.append(_node_pairs)
             steps.append(_pipeline)
@@ -223,7 +223,9 @@ def recursive_build_pipeline_from_cfg(  cfg,
             transformer.breakpoint = breakpoint
             transformer.print_task_result = pipeline_settings.print_task_result
             transformer.exchange_media = pipeline_settings.exchange_media
-            transformer.kafka_media = kafka_media
+            transformer.bootstrap_servers=pipeline_settings.bootstrap_servers
+            transformer.fernet_key = pipeline_settings.fernet_key
+            transformer.pipe_topic_name = pipeline_settings.pipe_topic_name
             
             # update steps list
             steps.append(transformer)
@@ -358,6 +360,20 @@ class Pipeline(sklearn_pipeline):
         return Xt
     
 
+def generate_fernet_key():
+    # create a fernet_key
+    fernet_key = Fernet.generate_key()
+    print("############# fernet_key", fernet_key)
+
+    return fernet_key
+
+def generate_pipe_topic_name():
+    pipeline_start_time = get_datetime_str(format="%Y-%m-%d %H:%M:%S")
+    sha256 = hashlib.sha256()
+    sha256.update(pipeline_start_time.encode("utf-8"))
+    pipe_topic_name =  f"pipeline_{sha256.hexdigest()}"
+    print(f"############ pipeline topic used: {pipe_topic_name}")
+    return pipe_topic_name
 
 class pipeline_settings():
     def __init__(self, cfg):
@@ -368,8 +384,11 @@ class pipeline_settings():
         self.delete_kafka_topic = self.pipeline_settings["delete_kafka_topic"] if "delete_kafka_topic" in self.pipeline_settings else 1
         self.bootstrap_servers = self.pipeline_settings["bootstrap_servers"] if "bootstrap_servers" in self.pipeline_settings else []
         self.default_type = self.pipeline_settings["module_type"] if "module_type" in self.pipeline_settings else ""
-        print(f"The default module type for this pipeline is set as {self.default_type}")
+        self.fernet_key = self.pipeline_settings["fernet_key"] if "fernet_key" in self.pipeline_settings else ""
+        self.pipe_topic_name = self.pipeline_settings["pipe_topic_name"] if "pipe_topic_name" in self.pipeline_settings else ""
+        # print(f"The default module type for this pipeline is set as {self.default_type}")
     
+
     def get_pipeline_settings_from_cfg(self):
         
         print_cfg = self.pipeline_settings.pop("print_cfg") if "print_cfg" in self.pipeline_settings else 0
@@ -400,17 +419,6 @@ class DataPipeline:
             print(cfg)
             print("".join(["-" for i in range(80)]), "----------------------", "".join(["-" for i in range(80)]))
 
-
-        self.kafka_media = None
-        if self._pipeline_settings.exchange_media=="kafka":
-            if self._pipeline_settings.bootstrap_servers:
-                from .kafka_media import kafkaMedia
-                # print("self._pipeline_settings.bootstrap_servers", self._pipeline_settings.bootstrap_servers)
-                self.kafka_media = kafkaMedia(bootstrap_servers=self._pipeline_settings.bootstrap_servers, topic="pipeline")
-                self.kafka_media.__enter__()
-                # print(self.kafka_media.producer, "#############################")
-
-
         self._cfg = cfg
         self._node_name_set=set()
         self._node_pairs = []
@@ -430,17 +438,6 @@ class DataPipeline:
         # self.nodes_text = init_nodes_text(self._nodename2transformer)
 
     def __enter__(self, ):
-        # tran = paramiko.Transport((self.host, int(self.port)))
-        # tran.connect(username=self.username, password=self.password)
-        # self.transport = tran
-
-        # try:        
-        #     sftp = paramiko.SFTPClient.from_transport(self.transport)
-        #     self.sftp = sftp
-        #     return self
-        # except:
-        #     self.transport.close()
-            # raise
         return self
 
     def __exit__(self, exc_type, exc_value, exc_tb):
@@ -466,8 +463,7 @@ class DataPipeline:
                                                                         self._index,
                                                                         self._parallel_index,
                                                                         self._nodename2transformer,
-                                                                        self._pipeline_settings,
-                                                                        self.kafka_media)
+                                                                        self._pipeline_settings)
 
         # setting first and last trans
         if _pipeline.steps:
